use piston_window::*;
use std::time::Instant;
use std::io;
mod apicontroller;
mod target;
mod test;


pub struct Game <'a>{
    targets: Vec<target::Target>,
    window: PistonWindow,
    mouse_x: f64,
    mouse_y: f64,
    points: u64,
    last_target: Instant,
    time_between_targets: f64,
    lives: u64,
    window_width: f64,
    window_height: f64,
    nick: &'a String
}
impl<'a> Game <'a>{

    pub fn new(window_width: f64, window_height: f64, nick: &'a String) -> Self {
        let targets = Vec::new();
        let s = [window_width, window_height];
        Self {
            targets,
            window: WindowSettings::new("game!", s)
                .exit_on_esc(true).build().unwrap(),
            mouse_x: 0.0,
            mouse_y: 0.0,
            points: 0,
            last_target: Instant::now(),
            time_between_targets: 2.0,
            lives: 3,
            window_width,
            window_height,
            nick
        }
    }
    fn draw(&mut self, event: &Event) -> () {
        self.window.draw_2d(event, |_context, graphics, _device| {
            clear([1.0; 4], graphics);
        });
        for i in 0..self.targets.len() {
            self.targets[i].draw(&mut self.window, &event);
        }
    }

    fn update(&mut self) -> () {
        for i in 0..self.targets.len() {
            self.targets[i].update();
            if !self.targets[i].is_inside_window(&self.window_width, &self.window_height) {
                self.targets.swap_remove(i);
                self.lives -= 1;
                break;
            }
        }
    }


    pub fn run(&mut self) -> () {
        while let Some(event) = self.window.next() {
            if self.lives < 1 {
                println!("You scored {} points", self.points);
                match apicontroller::ApiController::post_result(&self.nick, &self.points) {
                    Ok(()) => {
                        println!("Results send to server");
                    }
                    Err(err) => {
                        println!("Couldn't send results to server {:?}", err);
                    }
                }
                break;
            }
            Game::update(self);
            Game::draw(self, &event);

            event.mouse_cursor(|pos| {
                self.mouse_x = pos[0];
                self.mouse_y = pos[1];
            });

            if let Some(Button::Mouse(button)) = event.press_args() {
                if button == MouseButton::Left {
                    Game::check_hits(self)
                }
            }

            if self.last_target.elapsed().as_millis() as f64 >= self.time_between_targets * 1000.0 {
                Game::add_new_target(self);
                self.last_target = Instant::now();
                self.time_between_targets = self.time_between_targets-0.05;
                if self.time_between_targets < 0.3 {
                    self.time_between_targets = 0.3;
                }

            }
        }
    }

    fn add_new_target(&mut self) {
        self.targets.push(target::Target::generate_random(640))
    }

    fn check_hits(&mut self) {
        for i in 0..self.targets.len() {
            if self.targets[i].is_hit(&self.mouse_x, &self.mouse_y) {
                self.points += self.targets[i].get_points();
                self.targets.swap_remove(i);
                break;
            }
        }
    }
}

struct Menu {
    nick: String
}

impl Menu {

    fn new() -> Self {
        Self {
            nick: "Nick".to_string()
        }
    }

    fn run_first(&mut self)  {
        println!("What's your nick?");
        let mut nick = String::with_capacity(5);
        io::stdin().read_line(&mut nick).expect("Error reading input");
        println!("Hello {}", nick.as_str());
        self.nick = nick;
        Menu::run_loop(self);
    }

    fn run_loop(&mut self) {
        println!("Write 1 to play game or 2 to see the result or 3 to exit");
        let mut mode = String::with_capacity(5);
        io::stdin().read_line(&mut mode).expect("Error reading input");

        //game
        if mode.trim() == "1" {
            let mut game = Game::new(640.0, 420.0, &self.nick);
            game.run();
            drop(game);
            Menu::run_loop(self);
        }

        //ranking
        else if mode.trim() == "2" {
            match apicontroller::ApiController::get_best_user() {
                Ok(res) => {
                    if res.result == 0 {
                        println!("No best player yet");
                    }
                    else {
                        println!("Best player: {}, with {} points", res.nick, res.result);
                    }
                }
                Err(err) => {
                    println!("Couldn't get results from server {:?}", err);
                }
            }
            match apicontroller::ApiController::get_user_best(&self.nick) {
                Ok(res) => {
                    println!("Your ({}) best is {} points", res.nick, res.result);
                }
                Err(err) => {
                    println!("Couldn't get results from server {:?}", err);
                }
            }
            Menu::run_loop(self);
        }

        //exit
        else if mode.trim() == "3" {
            return;
        }

        //unknown command
        else {
            println!("No such option");
            Menu::run_loop(self);
        }
    }
}
fn main() {
    let mut menu = Menu::new();
    menu.run_first();
}


